-------------------------------------------------------------------------------------------------------
-- Rotacast_XChange - Exporting and Importing Rotacast Data
-------------------------------------------------------------------------------------------------------
-- Modules/GUI.lua - Setup Interface Tabs and Common Tools
--
-- Author: Expelliarm5s / November 2020 / All Rights Reserved
--
-- Version 1.0.1
-------------------------------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI StaticPopupDialogs StaticPopup_Show StaticPopup_Hide Rotacast

local addonName, addon = ...
local GUI = addon:NewModule("GUI", "AceConsole-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

GUI.display = nil   -- main interface frame

------------------------------------------------------------------------------
-- Debug Stuff

function GUI:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("GUI~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Create/Show Interface

StaticPopupDialogs[addonName .. "_ERROR"] = {
	text = "ERROR! Please review error message in the chat window!",
	button1 = ACCEPT,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 0,
}

function GUI:ShowError(msg)
	GUI:DebugPrintf("ShowError()")

	if not msg then
		return
	end
	addon:Printf(msg)
	StaticPopup_Show(addonName .. "_ERROR")
end

StaticPopupDialogs[addonName .. "_IMPORT"] = {
	text = "Are you SURE to import ?",
	button1 = ACCEPT,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 0,
}

function GUI:BtnImport()
	GUI:DebugPrintf("BtnImport()")

	local ebData
	if private.editframe then
		ebData = private.editframe:GetText()
	end

	if not ebData then
		return
	end

	if addon.db.global.promptForImport then
		StaticPopupDialogs[addonName .. "_IMPORT"].OnAccept = function()
			addon.GUI:DebugPrintf(addonName .. "_IMPORT")
			StaticPopup_Hide(addonName .. "_IMPORT")

			local result = addon.Data:Import(ebData)
			if result then
				GUI:ShowError(result)
			end
		end
		StaticPopup_Show(addonName .. "_IMPORT")
	else
		local result = addon.Data:Import(ebData)
		if result then
			GUI:ShowError(result)
		end
	end
end

function GUI:BtnExport()
	GUI:DebugPrintf("BtnExport()")

	local msg, ebData = addon.Data:Export()

	if ebData and private.editframe then
		private.editframe:SetText(ebData)
	else
		if msg then
			GUI:ShowError(msg)
		end
	end
end

-- create and return button/other widget
function GUI.AddButton(widgetType, label, status)
	GUI:DebugPrintf("AddButton %s, %s", widgetType, label)
	if not status then
		widgetType, label, status = "Button", widgetType, label
	end
	local button = assert(AceGUI:Create(widgetType))
	if button.SetText then
		button:SetText(tostring(label))
	end
	button:SetCallback("OnEnter", function() GUI:SetStatusLine(status) end)
	button:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	return button
end

function GUI:Load()
	GUI:DebugPrintf("Load()")

	-- create main window
	if not GUI.display then
		local display = AceGUI:Create("Frame")
		addon.db.global.ui = addon.db.global.ui or {}
		display:SetStatusTable(addon.db.global.ui)
		display.frame:SetFrameStrata("MEDIUM")

		GUI:DebugPrintf("  created the new widget %s", tostring(display))

		local title = "|cFF33FF99" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r"
		display:SetTitle(title)

		display:SetLayout("Fill")
		display:SetStatusText("--")
		display:EnableResize(true)
		display:SetCallback("OnClose", function(_display)
			GUI:DebugPrintf("OnClose for widget %s", tostring(_display))
			AceGUI:Release(_display)
			GUI.display = nil
			-- collectgarbage()
		end)

		GUI.display = display
	end

	if not GUI.display then
		return
	end

	if #GUI.display.children > 0 then
		GUI.display:ReleaseChildren()
	end

	local grp = AceGUI:Create("SimpleGroup")
	grp:SetLayout("Flow")

	-- import button
	local btnImport = AceGUI:Create("Button")
	btnImport:SetText("IMPORT")
	btnImport:SetRelativeWidth(1 / 2)
	btnImport:SetCallback("OnEnter", function() GUI:SetStatusLine("Import Data") end)
	btnImport:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	btnImport:SetCallback("OnClick", function()
		GUI:DebugPrintf("BtnImport_OnClick()")
		GUI:BtnImport()
	end)
	grp:AddChild(btnImport)

	-- export button
	local btnExport = AceGUI:Create("Button")
	btnExport:SetText("EXPORT")
	btnExport:SetRelativeWidth(1 / 2)
	btnExport:SetCallback("OnEnter", function() GUI:SetStatusLine("Import Data") end)
	btnExport:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	btnExport:SetCallback("OnClick", function()
		GUI:DebugPrintf("BtnExport_OnClick()")
		GUI:BtnExport()
	end)
	grp:AddChild(btnExport)

	local editGrp = AceGUI:Create("InlineGroup")
	editGrp:SetFullWidth(true)
	editGrp:SetFullHeight(true)
	editGrp:SetLayout("Fill")

	local editFrame = AceGUI:Create("RCXEditScrollFrame")
	editFrame:SetLayout("Fill")
	private.editframe = editFrame.content

	editGrp:AddChild(editFrame)
	grp:AddChild(editGrp)
	GUI.display:AddChild(grp)

	GUI.display:Show()

	return GUI.display
end

-----------------------------------------------------------------------------
-- Status Line

function GUI:SetStatusLine(txt)
	if GUI.display then
		GUI.display:SetStatusText(txt)
	end
end

-- EOF
